import os.path
import random
import sys
import time

from tkinter     import *
from tkinter.ttk import *

from curio import *

NUM_QUEUES = 4
DURATION   = 20     # number of seconds for which to run the message generators

MSGS = [
    "On your own head be it",
    "Well done!",
    "Scotland 3-1 England",
    "Gaun yersel'",
    "Let's put it to a vote",
    "You take the high road",
    "I'll take the low road",
]

ACTIONS = [
    'Panic',
    'Never mind',
    'Drink tea',
    'Drink whisky',
    'Cheer!',
    'Smile',
    'Cry',
]

class View(Treeview):

    def __init__(self, parent):
        super().__init__(
                parent,
                columns=['Item', 'Time', 'Message', 'Action'],
                displaycolumns=[1, 2, 3],
                height=15,
                padding=[4],
                selectmode='browse',
                show=['headings'])

        self.heading(1, text='Time', anchor=CENTER)
        self.heading(2, text='Message', anchor=W)
        self.heading(3, text='Action', anchor=W)

        self.column(1, anchor=CENTER, width=80)
        self.column(2, anchor=W, width=200)
        self.column(3, anchor=W, width=100)

        self.even = True
        self.tag_configure('evenrow', background='alice blue')
        self.tag_configure('oddrow', background='white')

    def append_row(self, row):
        tag = 'evenrow' if self.even else 'oddrow'
        self.even = not self.even
        self.see(self.insert('', 'end', values=row, tags=[tag]))

class Window(Frame):

    def __init__(self, parent):
        super().__init__(parent, padding=(5, 5, 5, 5))

        self.grid()

        # Create four table views
        self.views = []
        assert NUM_QUEUES == 4
        for row in range(2):
            for col in range(2):
                v = View(self)
                v.grid(column=col, row=row)
                self.views.append(v)

    def generate_message(self, ctrl):
        msg = random.choice(MSGS)
        action = random.choice(ACTIONS)
        stamp = time.strftime('%H:%M:%S')
        self.add_message( (ctrl, ('', stamp, msg, action)) )

    def add_message(self, args):
        ctrl, msg = args
        self.views[ctrl].append_row(msg)

class Application(Tk):
    def __init__(self, title):
        super().__init__()
        self.title(title)
        self._window = Window(self)
        self.running = Event()

    def destroy(self):
        self.running.clear()
        super().destroy()


#----------------------------------------------------------------------

async def generate_messages(app, ctrl, gap):
    await app.running.wait()
    async with ignore_after(DURATION):
        while app.running.is_set():
            app._window.generate_message(ctrl)
            await sleep(gap)

async def tk_runloop(app):
    await app.running.wait()
    while app.running.is_set():
        app.update()
        await sleep(0.1)

async def main():
    app = Application('curio/Tk Example')
    app.update()

    rl = await spawn(tk_runloop(app))

    m0 = await spawn(generate_messages(app, 0, 0.05))
    m1 = await spawn(generate_messages(app, 1, 0.1))
    m2 = await spawn(generate_messages(app, 2, 0.2))
    m3 = await spawn(generate_messages(app, 3, 0.4))

    await app.running.set()

    await rl.join()


#----------------------------------------------------------------------

if __name__ == '__main__':
    k = Kernel(with_monitor=True)
    k.run(main())
    k.run(None, shutdown=True)
